% function [parents] = children2ParentTree(Childs)
% 
% The input, Childs, is an (m-1)-by-2 matrix containing cluster tree information. 
% The leaf nodes in the cluster hierarchy are the objects in the original data set, numbered from 1 to m. 
% They are the singleton clusters from which all higher clusters are built. 
% Each newly formed cluster, corresponding to row i in Childs, is assigned the index m+i, where m is the total number of initial leaves.
% 
% Columns 1 and 2,Childs (i,1:2), contain the indices of the objects that were linked in pairs to form a new cluster. This new cluster is assigned the index value m+i. 
% There are m-1 higher clusters that correspond to the interior nodes of the hierarchical cluster tree. 
% 
% For example, consider a case with 30 initial nodes. 
% If the tenth cluster formed by the linkage function combines object 5 and object 7 and their distance is 1.5, then row 10 of Z will contain the values (5, 7, 1.5). 
% This newly formed cluster will have the index 10+30=40. 
% If cluster 40 shows up in a later row, 
% that means this newly formed cluster is being combined again into some bigger cluster.
% See linkage for more info
% vector of parent pointers, with p(i) = 0 for a root. (see treeplot for more info)

function [parents,indices] = children2ParentTree(childs)
parents=0; % first element
elements = length( childs(:,1)) +1;
indices = elements*2 - 1;
for ind  = 1 :elements -1;
    counter = elements*2 - ind;
    x = find (indices == counter);
    place = length(parents) + 1;%first
    parents(place)  = x;
    indices(place) = childs(elements-ind,1);
    place=place+1;%second
    parents(place)  = x;
    indices(place) = childs(elements-ind,2);
end